library(ComplexHeatmap)
library(ggplot2)
library(matrixStats)


for (f in list.files("./", "*feature_ranks.csv")) {
    print(f)
    data <- read.table(f, header=T, sep=",", stringsAsFactors=F, row.names=1)
    print(dim(data))
    data <- as.matrix(data[rowSums(abs(data) > 0) >= 2,])
    print(dim(data))
    annotation <- data.frame(sapply(colnames(data), function(x){return(strsplit(x, '_'))}))
    print(dim(annotation))
    rownames(annotation) = c("sample", "marker", "celltype")
    annotation <- data.frame(t(annotation))
    sample_color = unlist(sapply(1:6, function(x){gray((1+x)/7)}))
    names(sample_color) = c("BICCN2", "GSE111586", "GSE123576", "GSE126074", "GSE127257", "GSE1303990")
    col = list(marker=c('SF'='#E64B35FF', 'CU'='#4DBBD5FF', 'TA'='#00A087FF', 'TN'='#91D1C2FF', 'SC'='#3C5488FF', 'all'='darkgray'), sample=sample_color, celltype=c('IN'='darkgreen', 'EX'='darkorange', 'NN'='blue', 'P'='red', 'N'='blue'))
    ha <- HeatmapAnnotation(df=annotation, col=col, which="col")
    pdf(paste0(unlist(strsplit(f, '.', fixed=TRUE))[1], '_original_scale.pdf'))
    draw(Heatmap(data, name="Original", column_title="Samples", row_title="Genes", top_annotation=ha))
    #draw(Heatmap(data, name="Original", column_title="Samples", row_title="Genes"))
    dev.off()
    print(dim(data))
    data <- data[rowSums(abs(data[,annotation[,"marker"] != "all" & annotation[,"marker"] != "SF"])) >= 1,]
    print(dim(data))
    pdf(paste0(unlist(strsplit(f, '.', fixed=TRUE))[1], '_original_scale_limited.pdf'))
    draw(Heatmap(scale(data), name="Original", column_title="Samples", row_title="Genes", top_annotation=ha))
    dev.off()
    tdata <- sign(as.matrix(data))
    pdf(paste0(unlist(strsplit(f, '.', fixed=TRUE))[1], '_rank_scale.pdf'))
    draw(Heatmap(tdata, name="Ranking", column_title="Samples", row_title="Genes", top_annotation=ha))
    dev.off()
}
